@@ -17,7 +17,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: AudioClip.cpp,v 1.32 2006/07/03 17:51:56 r_sijrier Exp $
+$Id: AudioClip.cpp,v 1.35 2006/07/07 15:01:41 r_sijrier Exp $
 */
 
 #include <cfloat>
@@ -143,6 +143,16 @@ int AudioClip::set_state(const QDomNode& node )
 	sourceEndFrame = sourceStartFrame + m_length;
 	set_track_start_frame( e.attribute( "trackstart", "" ).toUInt());
 	
+/*	QDomElement CurvesNode = node.firstChildElement("Curves");
+	if (!CurvesNode.isNull()) {
+		QDomNode CurveNode = CurvesNode.firstChild();
+		while (!CurveNode.isNull()) {
+			QDomElement e = CurveNode.toElement();
+			Curve* curve = new Curve(e);
+			CurveNode = CurveNode.nextSibling();
+		}
+	}*/
+	
 	return 1;
 }
 
@@ -170,6 +180,12 @@ QDomNode AudioClip::get_state( QDomDocument doc )
 		node.setAttribute(sourceName, readSources.at(i)->get_id());
 	}
 
+// 	QDomNode curves = doc.createElement("Curves");
+// 	
+// 	curves.appendChild(fadeIn->get_state(doc));
+// 	
+// 	node.appendChild(curves);
+
 	return node;
 }
 
@@ -344,6 +360,9 @@ int AudioClip::set_selected(bool selected)
 	return 1;
 }
 
+//
+//  Function called in RealTime AudioThread processing path
+//
 int AudioClip::process(nframes_t nframes, audio_sample_t* mixdown, uint channel)
 {
 	if (isRecording) {
@@ -356,10 +375,9 @@ int AudioClip::process(nframes_t nframes, audio_sample_t* mixdown, uint channel)
 		return -1;	// Channel doesn't exist!!
 	}
 
-
 	nframes_t mix_pos;
 	
-	if ( (trackStartFrame < m_song->get_transport_frame()) && (trackEndFrame > m_song->get_transport_frame()) ) {
+	if ( (trackStartFrame <= m_song->get_transport_frame()) && (trackEndFrame > (m_song->get_transport_frame())) ) {
 		mix_pos = m_song->get_transport_frame() - trackStartFrame + sourceStartFrame;
 	} else {
 		return 0;
@@ -380,20 +398,26 @@ int AudioClip::process(nframes_t nframes, audio_sample_t* mixdown, uint channel)
 		read_frames = readSources.at(channel)->file_read(mixdown, mix_pos, nframes);
 	}
 
-	if (read_frames == 0)
+	
+	if (read_frames == 0) {
 		return 0;
+	}
 
 	
 	audio_sample_t* gainbuffer = m_song->gainbuffer;
 	
 	// FADE IN
 	
-	if (m_song->get_transport_frame() < (trackStartFrame + fadeIn->get_range()) ) {
+	if (m_song->get_transport_frame() < (trackStartFrame + fadeIn->get_range())) {
 // 		printf("mix_pos is %d, len is %d\n", mix_pos, fadeIn->get_range());
 		nframes_t limit;
 
 		limit = std::min (read_frames, (uint)fadeIn->get_range());
 		
+		if (fadeIn->get_range() < 16) {
+			limit = 0;
+		}
+		
 		int fadepos = m_song->get_transport_frame() - trackStartFrame;
 		
 		fadeIn->get_vector ( fadepos, fadepos + limit, gainbuffer, limit);
@@ -406,12 +430,16 @@ int AudioClip::process(nframes_t nframes, audio_sample_t* mixdown, uint channel)
 	
 	// FADE OUT
 	
-	if (m_song->get_transport_frame() > (trackEndFrame - fadeOut->get_range()) ) {
+	if (m_song->get_transport_frame() > (trackEndFrame - fadeOut->get_range())) {
 		
 		nframes_t limit;
 		
 		limit = std::min (read_frames, (uint)fadeOut->get_range());
 		
+		if (fadeOut->get_range() < 16) {
+			limit = 0;
+		}
+		
 		int fadepos = m_song->get_transport_frame() - (trackEndFrame - (nframes_t)fadeOut->get_range());
 		
 		fadeOut->get_vector (fadepos, fadepos + limit, gainbuffer, limit);
@@ -426,6 +454,9 @@ int AudioClip::process(nframes_t nframes, audio_sample_t* mixdown, uint channel)
 	return 1;
 }
 
+//
+//  Function called in RealTime AudioThread processing path
+//
 void AudioClip::process_capture( nframes_t nframes, uint channel )
 {
 	WriteSource* source = writeSources.at(channel);
@@ -618,6 +649,8 @@ void AudioClip::add_audio_source( ReadSource* rs, int channel )
 	rate = rs->get_rate();
 	
 	set_sources_active_state();
+
+	rs->set_audio_clip(this);
 	
 	emit stateChanged();
 }
@@ -630,7 +663,13 @@ void AudioClip::finish_write_source( WriteSource * ws )
 	
 	for (int i=0; i<writeSources.size(); ++i) {
 		if (ws == writeSources.at(i)) {
+		
+			//FIXME. The audiosourcesmanager is actually in charge of creating
+			// new ReadSources!!!!!!
+			// So we should move this code into asm, and get the source from there again :-)
+			// As a temp. fix, we call rs->ref() here to not crash :-(
 			ReadSource* rs = new ReadSource(ws);
+			rs->ref();
 			
 			rs->set_created_by_song( m_song->get_id() );
 			rs->set_original_bit_depth( audiodevice().get_bit_depth() );
@@ -790,8 +829,8 @@ void AudioClip::set_fade_in_shape( FadeShape shape, nframes_t len )
 
 	fadeInShape = shape;
 	
-	if (fadeIn->nodes.size() > 0) {
-		THREAD_SAVE_ADD(fadeIn, fadeIn, private_clear() );
+	if (fadeIn->get_nodes()->size() > 0) {
+		THREAD_SAVE_CALL(fadeIn, private_clear(), fadeIn);
 	}
 	
 	switch (shape) {
@@ -848,8 +887,8 @@ void AudioClip::set_fade_out_shape( FadeShape shape, nframes_t len )
 {
 	fadeOutShape = shape;
 	
-	if (fadeOut->nodes.size() > 0) {
-		THREAD_SAVE_ADD(fadeOut, fadeOut, private_clear());
+	if (fadeOut->get_nodes()->size() > 0) {
+		THREAD_SAVE_CALL(fadeOut, private_clear(), fadeOut);
 	}
 	
 	switch (shape) {
@@ -953,8 +992,8 @@ void AudioClip::calculate_normalization_factor(float targetdB)
 		target -= FLT_EPSILON;
 	}
 
-	fpos = 0;
-	fend = sourceLength;
+	fpos = sourceStartFrame;
+	fend = sourceEndFrame;
 
 	/* first pass: find max amplitude */
 
