@@ -17,7 +17,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: Song.cpp,v 1.26 2006/07/03 17:51:56 r_sijrier Exp $
+$Id: Song.cpp,v 1.28.2.2 2006/07/31 13:10:09 r_sijrier Exp $
 */
 
 #include <QTextStream>
@@ -41,13 +41,12 @@ $Id: Song.cpp,v 1.26 2006/07/03 17:51:56 r_sijrier Exp $
 #include "MtaRegion.h"
 #include "MtaRegionList.h"
 #include "Peak.h"
-//#include "AudioPluginChain.h"
-#include "AudioPluginSelector.h"
 #include "Export.h"
 #include "DiskIO.h"
 #include "WriteSource.h"
 #include "AudioClipManager.h"
 #include "Tsar.h"
+#include "SnapList.h"
 
 #include "LocatorView.h"
 #include "ContextItem.h"
@@ -159,6 +158,7 @@ void Song::init()
 	changed = rendering = false;
 	firstVisibleFrame=workingFrame=activeTrackNumber=0;
 	trackCount = 0;
+	snaplist = new SnapList(this);
 }
 
 int Song::set_state( const QDomNode & node )
@@ -175,7 +175,6 @@ int Song::set_state( const QDomNode & node )
 	set_gain(e.attribute( "mastergain", "1.0").toFloat() );
 	set_hzoom(e.attribute( "hzoom", "" ).toInt());
 	set_first_visible_frame(e.attribute( "firstVisibleFrame", "0" ).toUInt());
-	set_work_at(e.attribute( "workingFrame", "0").toUInt());
 	
 	int trackBaseY = LocatorView::LOCATOR_HEIGHT;
 
@@ -201,6 +200,9 @@ int Song::set_state( const QDomNode & node )
 		trackNode = trackNode.nextSibling();
 	}
 
+	// Set work at will trigger a seek, which in turn get's all 
+	// audio buffers into proper pre-filled state!
+	set_work_at(e.attribute( "workingFrame", "0").toUInt());
 	return 1;
 }
 
@@ -284,7 +286,7 @@ void Song::add_track(Track* track)
 		emit trackAdded(track);
 // 		printf("Song is not transporting, save to add Track directly\n");
 	} else {
-		THREAD_SAVE_ADD_EMIT_SIGNAL(this, track, private_add_track(Track*), trackAdded(Track*));
+		THREAD_SAVE_CALL_EMIT_SIGNAL(this, track, private_add_track(Track*), trackAdded(Track*));
 	}
 	
 }
@@ -295,7 +297,7 @@ void Song::remove_track(Track* track)
 		private_remove_track(track);
 		emit trackRemoved(track);
 	} else {
-		THREAD_SAVE_REMOVE_EMIT_SIGNAL(this, track, private_remove_track(Track*), trackRemoved(Track*));
+		THREAD_SAVE_CALL_EMIT_SIGNAL(this, track, private_remove_track(Track*), trackRemoved(Track*));
 	}
 
 }
@@ -493,19 +495,24 @@ int Song::snapped_x(int x)
 {
 	int nx = x;
 	if (isSnapOn) {
-		int wx = frame_to_xpos(workingFrame);
-		// TODO check if it is close to upper or lower nearest clip's edges
-		if (nx<10) // check if it is close to track begin
-			nx=0;
-		if (abs(nx-wx)<10) // first try to snap to working block
-		{
-			PMESG("snapping !");
-			nx=wx;
-		}
+		nx = snaplist->get_snap_value(x);
 	}
 	return nx;
 }
 
+// recalculate the snap positions. SnapList must know
+// which clip is being moved, in order to exclude
+// it from the snap list. It's much more convenient that way
+void Song::update_snaplist(AudioClip *u_clip)
+{
+	snaplist->update_snaplist(u_clip);
+}
+
+SnapList* Song::get_snap_list()
+{
+	return snaplist;
+}
+
 nframes_t Song::xpos_to_frame(int xpos)
 {
 	return (firstVisibleFrame + xpos * Peak::zoomStep[m_hzoom]);
@@ -575,18 +582,33 @@ void Song::set_first_visible_frame(nframes_t pos)
 void Song::set_work_at(nframes_t pos)
 {
 	PMESG2("entering set_work_at");
-	newTransportFramePos = pos;
-	workingFrame = pos;
 
+/** use this part if the work cursor should _not_ snap **/
+// 	newTransportFramePos = pos;
+// 	workingFrame = pos;
+/** use this if it should snap **/
+	snaplist->update_snaplist();
+	long position = xpos_to_frame(snapped_x(frame_to_xpos(pos)));
+	workingFrame = xpos_to_frame(snapped_x(frame_to_xpos(pos)));
+/** **/
+
+	Q_ASSERT(position >= 0);
+	
+	newTransportFramePos = (uint) position;
+	
 	// If there is no transport, start_seek() will _not_ be
 	// called from within process(). So we do it now!
-	if (!transport)
+	if (!transport) {
 		start_seek();
+	}
 
 	seeking = true;
 	emit workingPosChanged();
 }
 
+//
+//  Function _could_ be called in RealTime AudioThread processing path
+//
 void Song::start_seek()
 {
 	PMESG2("Song :: entering start_seek");
@@ -600,7 +622,8 @@ void Song::start_seek()
 
 	diskio->prepare_for_seek();
 
-	emit seekStart(newTransportFramePos);
+	THREAD_SAVE_EMIT_SIGNAL(this, seekStart(uint), newTransportFramePos);
+	
 	PMESG2("Song :: leaving start_seek");
 }
 
@@ -766,20 +789,50 @@ Command* Song::set_curve_mode()
 void Song::solo_track(Track* t)
 {
 	bool wasSolo = t->is_solo();
-	
+
+	t->set_muted_by_solo(!wasSolo);
+	t->set_solo(!wasSolo);
+
+	bool hasSolo = false;
 	foreach(Track* track, m_tracks) {
-		track->set_solo(false);
+		track->set_muted_by_solo(!track->is_solo());
+		if (track->is_solo()) hasSolo = true;
+	}
+
+	if (!hasSolo) {
+		foreach(Track* track, m_tracks) {
+			track->set_muted_by_solo(false);
+		}
+	}
+}
+
+Command* Song::toggle_solo()
+{
+	bool hasSolo = false;
+	foreach(Track* track, m_tracks) {
+		if (track->is_solo()) hasSolo = true;
+	}
+
+	foreach(Track* track, m_tracks) {
+		track->set_solo(!hasSolo);
 		track->set_muted_by_solo(false);
 	}
-	
-	if (!wasSolo) {
-		foreach(Track* track, m_tracks) {
-			track->set_muted_by_solo(true);
-		}
-		t->set_solo(true);
+
+	return (Command*) 0;
+}
+
+Command *Song::toggle_mute()
+{
+	bool hasMute = false;
+	foreach(Track* track, m_tracks) {
+		if (track->is_muted()) hasMute = true;
 	}
-	
-	
+
+	foreach(Track* track, m_tracks) {
+		track->set_muted(!hasMute);
+	}
+
+	return (Command*) 0;
 }
 
 Command* Song::work_next_edge()
@@ -844,6 +897,9 @@ void Song::set_hzoom( int hzoom )
 	emit hzoomChanged();
 }
 
+//
+//  Function called in RealTime AudioThread processing path
+//
 int Song::process( nframes_t nframes )
 {
 	// If no need for playback/record, return.
@@ -852,7 +908,7 @@ int Song::process( nframes_t nframes )
 		return 0;
 		
 	if (stopTransport) {
-		emit transferStopped();
+		THREAD_SAVE_EMIT_SIGNAL(this, transferStopped(), 0);
 		transport = false;
 		realtimepath = false;
 		stopTransport = false;
@@ -860,8 +916,7 @@ int Song::process( nframes_t nframes )
 		return 0;
 	}
 
-	// zero the masterOut buffers _and_ the mixdown buffer which is used 
-	// in the Tracks' to mixdown the audioclips!
+	// zero the masterOut buffers
 	masterOut->silence_buffers(nframes);
 	
 	int processResult = 0;
