@@ -20,7 +20,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 
-$Id: AlsaDriver.cpp,v 1.3 2006/06/16 14:07:38 r_sijrier Exp $
+$Id: AlsaDriver.cpp,v 1.5 2006/07/27 18:45:24 r_sijrier Exp $
 */
 
 
@@ -187,6 +187,8 @@ int AlsaDriver::setup()
 				printf ("ALSA Driver: You do not have permission to open the audio device \"%s\" for playback\n", playback_pcm_name);
 				return -1;
 				break;
+			default:
+				PERROR ("snd_pcm_open(playback_handle, ..) failed with unknown error type");
 			}
 
 			playback_handle = 0;
@@ -210,6 +212,8 @@ int AlsaDriver::setup()
 				PWARN ("ALSA Driver: You do not have permission to open the audio device \"%s\" for capture", capture_pcm_name);
 				return -1;
 				break;
+			default:
+				PERROR ("snd_pcm_open(capture_handle, ...) failed with unknown error type");
 			}
 
 			capture_handle = 0;
@@ -1195,6 +1199,7 @@ again:
 		if (need_playback) {
 			for (i = 0; i < playback_nfds; i++) {
 				if (pfd[i].revents & POLLERR) {
+					PWARN("playback pollerror, xrun_detected is true");
 					xrun_detected = true;
 				}
 
@@ -1227,6 +1232,7 @@ again:
 		if (need_capture) {
 			for (i = ci; i < nfds; i++) {
 				if (pfd[i].revents & POLLERR) {
+					PWARN("capture pollerror, xrun_detected is true");
 					xrun_detected = true;
 				}
 
@@ -1266,6 +1272,7 @@ again:
 	if (capture_handle) {
 		if ((capture_avail = snd_pcm_avail_update (capture_handle)) < 0) {
 			if (capture_avail == -EPIPE) {
+				PWARN("capture pipeerror, xrun_detected is true");
 				xrun_detected = true;
 			} else {
 				PERROR ("ALSA Driver: unknown avail_update return value (%ld)", capture_avail);
@@ -1279,6 +1286,7 @@ again:
 	if (playback_handle) {
 		if ((playback_avail = snd_pcm_avail_update (playback_handle)) < 0) {
 			if (playback_avail == -EPIPE) {
+				PWARN("playback pipeerror, xrun_detected is true");
 				xrun_detected = true;
 			} else {
 				PERROR ("ALSA Driver: unknown avail_update return value (%ld)", playback_avail);
