@@ -20,6 +20,9 @@
     $Id: PanelLed.h,v 1.2 2006/05/01 21:31:58 r_sijrier Exp $
 */
 
+#ifndef PANEL_LED_VIEW_H
+#define PANEL_LED_VIEW_H
+
 #include "ViewItem.h"
 #include <QString>
 
@@ -52,4 +55,6 @@ public slots:
         void schedule_for_repaint();
 };
 
+#endif
+
 //eof
